
/**
 * PHP number_format (http://kevin.vanzonneveld.net)
 *
 * @param  {[type]} number        [description]
 * @param  {[type]} decimals      [description]
 * @param  {[type]} dec_point     [description]
 * @param  {[type]} thousands_sep [description]
 * @return {[type]}               [description]
 */
var number_format = function(number, decimals, dec_point, thousands_sep) {

  // Strip all characters but numerical ones.
  number = (number + '').replace(/[^0-9+\-Ee.]/g, '');
  var n = !isFinite(+number) ? 0 : +number,
    prec = !isFinite(+decimals) ? 0 : Math.abs(decimals),
    sep = (typeof thousands_sep === 'undefined') ? ',' : thousands_sep,
    dec = (typeof dec_point === 'undefined') ? '.' : dec_point,
    s = '',
    toFixedFix = function (n, prec) {
      var k = Math.pow(10, prec);
      return '' + Math.round(n * k) / k;
    };
  // Fix for IE parseFloat(0.55).toFixed(0) = 0;
  s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.');
  if (s[0].length > 3) {
    s[0] = s[0].replace(/\B(?=(?:\d{3})+(?!\d))/g, sep);
  }
  if ((s[1] || '').length < prec) {
    s[1] = s[1] || '';
    s[1] += new Array(prec - s[1].length + 1).join('0');
  }
  return s.join(dec);
};


/*
 * Close/Destroy method for Backbone views
 */
Backbone.View.prototype.close = function () {

    this.remove();
    this.trigger('close');
    this.off();
    this.undelegateEvents();

    if (this.onClose) {
        this.onClose();
    }

    return true;
};

/*
 * Hide method for Backbone views.
 */
Backbone.View.prototype.hide = function() {

    this.$el.hide();
    this.trigger('hide');

    return true;
};

/*
 * Show method for Backbone views.
 */
Backbone.View.prototype.show = function() {

    this.$el.show();
    this.trigger('show');

    return true;
};

/*
 * Current VAT (MwSt) value.
 */
var VAT = 19;

/*
 * List of common accessories.
 */
var ACCS_LIST = [
    {
        value: 'D01_2',
        label: 'Schiebdach',
        group: 'XXX'
    },{
        value: 'E11',
        label: 'Einparkhilfe',
        group: 'XXX'
    },{
        value: 'E12',
        label: 'Navigationssystem',
        group: 'XXX'
    },{
        value: 'E13',
        label: 'Tempomat',
        group: 'XXX'
    },{
        value: 'F15',
        label: 'Servolenkung',
        group: 'XXX'
    },{
        value: 'F01',
        label: 'ABS',
        group: 'XXX'
    },{
        value: 'F06',
        label: 'Allrad',
        group: 'XXX'
    },{
        value: 'F07',
        label: 'Elektronische Stabilitätsprogramm (ESP)',
        group: 'XXX'
    },{
        value: 'F12',
        label: 'Leichtmetallfelgen',
        group: 'XXX'
    },{
        value: 'F31',
        label: 'Partikelfilter',
        group: 'XXX'
    },{
        value: 'I04_1',
        label: 'Leder',
        group: 'interior'
    },{
        value: 'I04_3',
        label: 'Velour',
        group: 'interior'
    },{
        value: 'I04_4',
        label: 'Stoff',
        group: 'interior'
    },{
        value: 'I04_5',
        label: 'Teilleder',
        group: 'interior'
    },{
        value: 'I04_6',
        label: 'Alcantara',
        group: 'interior'
    },{
        value: 'I04_7',
        label: 'Andere',
        group: 'interior'
    },{
        value: 'K01_1',
        label: 'Klimaanlage',
        group: 'XXX'
    },{
        value: 'K02',
        label: 'Klimaautomatik / Klimatronic',
        group: 'XXX'
    },{
        value: 'K05',
        label: 'El. Fensterheber',
        group: 'XXX'
    },{
        value: 'K07',
        label: 'Sitzheizung',
        group: 'XXX'
    },{
        value: 'K11',
        label: 'Standheizung',
        group: 'XXX'
    },{
        value: 'K15',
        label: 'keine Klimaanlage/-automatik',
        group: 'XXX'
    },{
        value: 'S01',
        label: 'Xenon',
        group: 'XXX'
    },{
        value: 'S12',
        label: 'Fahrerairbag',
        group: 'XXX'
    },{
        value: 'S13',
        label: 'Beifahrerairbag',
        group: 'XXX'
    },{
        value: 'S14',
        label: 'Fondairbag',
        group: 'XXX'
    },{
        value: 'S15',
        label: 'Airbag Seite (vorn)',
        group: 'XXX'
    },{
        value: 'S16',
        label: 'Airbag Seite (hinten)',
        group: 'XXX'
    },{
        value: 'S17',
        label: 'Kopfairbags',
        group: 'XXX'
    },{
        value: 'S25_2',
        label: 'El. Wegfahrsperre',
        group: 'XXX'
    },{
        value: 'V01',
        label: 'metalic',
        group: 'XXX'
    },{
        value: 'V02',
        label: 'Anhängerkupplung',
        group: 'XXX'
    },{
        value: 'V28',
        label: 'Zentralveriegelung',
        group: 'XXX'
    },{
        value: '0023',
        label: 'Sonnendach',
        group: 'xxx'
    },{
        value: '0087',
        label: 'Spurwechselassistent',
        group: 'XXX'
    }
];

/*
 *
 *
 */
var app = (function() {

    var _app = _.extend({}, Backbone.Events);

    _app.isLoading = 0;

    /*
     * App started loading data.
     */
    _app.on('loading-started', function() {

        _app.isLoading++;

        if (_app.isLoading > 0) {
            $('#isLoading').show();
        }
    });

    /*
     * App finished loading data.
     */
    _app.on('loading-finished', function() {

        if (_app.isLoading === 0) {
            return true;
        }

        _app.isLoading--;

        if (_app.isLoading === 0) {
            $('#isLoading').hide();
        }
    });


    var PAGIN_MAX_ITEMS_PER_PAGE = 5;

    /*
     * ------------------- V E H I C L E   F I L T E R
     */

    var BaseFilterParam = Backbone.View.extend({

        tagName: 'p',
        className: 'base',

        name:  '',
        label: 'label',
        items: [],
        input: 'select', // select | input (radio)

        events: {

            'click input[type="radio"]': function(e) {
                this.trigger('change', $(e.target).val());
            },

            'change select': function(e) {
                this.trigger('change', $(e.target).val());
            }
        },

        initialize: function(options) {

            _.bindAll(this, 'render', 'refreshItems');

            options = options || {};

            if (!_(options).has('name')) {
                throw new Error('No name for filter.');
            }

            this.name = options.name;

            if (!_(options).has('label')) {
                throw new Error('No label for filter.');
            }

            this.label = options.label;

            if (_(options).has('input') && options.input != 'select' && options.input != 'radio') {
                throw new Error('Invalid input for filter: ' + options.input);
            }
            else {
                this.input = options.input;
            }

            if (_(options).has('items')) {

                if (false === _(options.items).isArray()) {
                    throw new Error('Invalid items for filter: ' + options.items);
                }

                this.items = options.items;
            }

            this.render();
        },

        render: function() {

            var that = this;
            var $input = null;

            /*
             * Radio param
             */
            if (that.input == 'radio') {

                var inputs = [];
                _(that.items).each(function(item) {
                    inputs.push('<input type="radio" filter-type="BaseValue" id="' + that.name + '_' + item.Key + '" name="' + that.name + '" value="' + item.Key + '"/>' +
                                '<label class="radio" for="' + that.name + '_' + item.Key + '">' + item.Value + '</label>');
                });

                $input = $('<label>' + that.label + '</label><span>' + inputs.join('') + '</span>');
            }

            /*
             * Select param
             */
            else {

                var options = ['<option value="0">- - -</option>'];

                _(that.items).each(function(item) {

                    if (item.Key === 0) {
                        return true;
                    }

                    options.push('<option value="' + item.Key + '">' + item.Value + '</option>');
                });

                $input = $('<label>' + that.label + '</label><select name="' + that.name + '" filter-type="BaseValue">' + options.join() + '</select>');
            }

            if (that.items.length === 0) {

                that.$el.addClass('disabled');
                $input.attr('disabled', 'disabled');
            }
            else {

                that.$el.removeClass('disabled');
                $input.removeAttr('disabled');
            }

            that.$el.empty().append($input);

            /*
             * Auto select radio input if only 1 item is available.
             */
            if (that.input == 'radio' && that.items.length == 1) {
                that.$el.find('input[type="radio"]').click();
            }

            return this;
        },

        refreshItems: function(items) {
            this.items = items;
            this.render();
        }

    });

    var MinMaxFilterParam = Backbone.View.extend({

        tagName: 'p',
        className: 'minmax',

        name:  '',
        label: 'label',
        items: [],

        events: {
            'change select': function(e) {
                this.trigger('change', $(e.target).val(), $(e.target).attr('filter-type'));
            }
        },

        initialize: function(options) {

            _.bindAll(this, 'render', 'refreshItems');

            options = options || {};

            if (!_(options).has('name')) {
                throw new Error('No name for filter.');
            }

            this.name = options.name;

            if (!_(options).has('label')) {
                throw new Error('No label for filter.');
            }

            this.label = options.label;

            if (_(options).has('items')) {

                if (false === _(options.items).isArray()) {
                    throw new Error('Invalid items for filter: ' + options.items);
                }

                this.items = options.items;
            }

            this.render();
        },

        render: function() {

            var that = this;

            $label = $('<label>' + that.label + '</label>');
            $select_min = $('<select name="' + that.name + '" filter-type="MinValue"></select>');
            $select_max = $('<select name="' + that.name + '" filter-type="MaxValue"></select>');

            var options = ['<option value="0">- - -</option>'];
            _(that.items).each(function(item) {

                if (item.Key === 0) {return true;}
                options.push('<option value="' + item.Key + '">' + item.Value + '</option>');
            });

            $select_min.append(options.join(''));
            $select_max.append(options.join(''));

            if (that.items.length === 0) {
                that.$el.addClass('disabled');
                $select_min.attr('disabled', 'disabled');
                $select_max.attr('disabled', 'disabled');
            }
            else {
                that.$el.removeClass('disabled');
                $select_min.removeAttr('disabled');
                $select_max.removeAttr('disabled');
            }


            that.$el
                .empty()
                .append(
                    $label,
                    $select_min,
                    $select_max
                );

            return this;
        },

        refreshItems: function(items) {
            this.items = items;
            this.render();
        }
    });

    var CheckFilterParam = Backbone.View.extend({

        tagName: 'p',
        className: 'check',

        label: 'label',
        value: null,
        group: 'misc',

        events: {
            'click input[type="checkbox"]': function(e) {

                var $target = $(e.target);

                if (true === $target.prop('checked')) {
                    this.trigger('check', $target.attr('name'));
                }
                else {
                    this.trigger('uncheck', $target.attr('name'));
                }
            }
        },

        initialize: function(options) {

            _.bindAll(this, 'render', 'enable', 'disable');

            options = options || {};

            // if (!_(options).has('name')) {
            //     throw new Error('No name for filter.');
            // }

            if (!_(options).has('label')) {
                throw new Error('No label for filter.');
            }

            if (!_(options).has('value')) {
                throw new Error('No value for filter.');
            }

            // this.name  = options.name;
            this.label = options.label;
            this.value = options.value;

            if (_(options).has('group')) {
                this.group = options.group;
            }

            this.render();
        },

        render: function() {

            var that = this;

            $label = $('<label for="acc_' + that.value + '">' + that.label + '</label>');
            $input = $('<input type="checkbox" disabled="disabled" id="acc_' + that.value + '" name="' + that.value + '" value="' + that.value + '"/>');

            that.$el
                .empty()
                .append($input, $label)
                .addClass(that.name)
                .addClass('disabled');

            return this;
        },

        enable: function() {

            this.$el
                .removeClass('disabled')
                .find('input')
                    .removeAttr('disabled');

        },

        disable: function() {

            this.$el
                .addClass('disabled')
                .find('input')
                    .attr('disabled', 'disabled')
                    .prop('checked', false);
        }
    });


    var AccsListView = Backbone.View.extend({

        className: 'accs_list',

        initialize: function(options) {

            var that = this;
                that.accs_list = [];

            _.bindAll(this, 'render', 'enableItems');

            if (_(options).has('items') && _(options.items).isArray()) {

                _(options.items).each(function(item) {

                    var accs_item = new CheckFilterParam(item);

                    accs_item.on('check', function(name) {
                        that.trigger('change.add', name);
                    });

                    accs_item.on('uncheck', function(name) {
                        that.trigger('change.remove', name);
                    });

                    that.accs_list.push(accs_item);
                });
            }

            this.render();
        },

        render: function() {

            var that = this;

            that.$el.empty();

            _(that.accs_list).each(function(accs_list_item) {
                that.$el.append(accs_list_item.$el);
            });


            return that;
        },

        enableItems: function(enable_items) {

            var that = this;

            enable_items = enable_items || {};

            if (!_(enable_items).isArray()) {
                enable_items = [enable_items];
            }

            _(that.accs_list).each(function(accs_list_item) {

                if (_(enable_items)
                        .find(function(enable_item) {
                            return enable_item == accs_list_item.value;
                        })) {

                    accs_list_item.enable();
                }
                else {
                    accs_list_item.disable();
                }
            });
        }
    });


    var FilterModel = Backbone.Model.extend({

        defaults: {
            BaseValue: {},
            MinValue: {},
            MaxValue: {}
        },

        page_num: 1,

        limit: PAGIN_MAX_ITEMS_PER_PAGE,

        initialize: function() {
            _.bindAll(this, 'group', 'groupAll', 'isSetInFilter', 'filter', 'getFilteredList',
                            'addAccsItem', 'removeAccsItem', 'prepareResult');
        },

        group: function(fields) {

            var that = this;
            var _group = function(field) {

                _app.trigger('loading-started');
                $.post('data/group.php',

                    {
                        filter: that.toJSON(),
                        field: field
                    },

                    function(response) {
                        _app.trigger('loading-finished');
                        that.trigger('grouped.' + field, response);
                    },'json')

                    .error(function(response) {
                        _app.trigger('loading-finished');
                        console.debug('group error ', response);
                    });
            };

            if (!_(fields).isArray()) {
                fields = [fields];
            }

            _(fields).each(function(single_field) {

                if (false === that.isSetInFilter(single_field)) {
                    _group(single_field);
                }

            });
        },

        groupAll: function() {

            var that = this;

            var group_fields = [
                'Area',
                // 'Category',
                'Manufacturer',
                'Model',
                'Fuel',
                'Gearbox',
                'RegDate',
                'Km',
                'Power',
                'Price.Client',
                'AccsList'
            ];

            /*
             * Remove Model from list if Manufacturer is not selected.
             */
            group_fields = _(group_fields).reject(function(field_name) {
                return (field_name == 'Model') && (typeof that.get('BaseValue').Manufacturer == 'undefined' || that.get('BaseValue').Manufacturer.Id == '0');
            });

            this.group(group_fields);

            return true;
        },


        reset: function(field_name) {

            var that = this;
            var filter = that.toJSON();
            var filter_keys = _(filter).keys();

            for (var i = 0, l = filter_keys.length; i < l; i++) {

                var filter_key = filter_keys[i];

                if (_(filter[filter_key]).has(field_name)) {

                    delete filter[filter_key][field_name];
                    that.trigger('reset.' + field_name);

                    return true;
                }
            }

            return false;
        },

        isSetInFilter: function (field_name) {

            var that = this;
            var filter = that.toJSON();
            var filter_keys = _(filter).keys();

            field_name = field_name.split('.');

            for (var i = 0, l = filter_keys.length; i < l; i++) {

                var filter_key = filter_keys[i];

                if (_(filter[filter_key]).has(field_name[0])) {

                    if (1 < field_name.length) {
                        return _(filter[filter_key][field_name[0]]).has(field_name[1]);
                    }

                    return true;
                }
            }

            return false;
        },

        filter: function(fields) {

            var that = this;
            fields = fields || [];

            var pagination = {
                page_num: that.page_num,
                limit: that.limit
            };

            _app.trigger('loading-started');
            $.post('data/filter.php',

                {
                    filter: that.toJSON(),
                    fields: fields,
                    pagination: pagination
                },

                function(response) {
                    _app.trigger('loading-finished');
                    that.trigger('filtered', that.prepareResult(response));
                }, 'json')

                .error(function(response) {
                    _app.trigger('loading-finished');
                    console.debug('filter error ', response);
                });

            return true;
        },

        prepareResult: function(response) {

            _(response.items).each(function(item) {

                item.PowerPs = Number(item.Power * 1.359621617).toFixed(0);
                item.RegDate = item.RegDate.substr(5, 2) + '/' + item.RegDate.substr(0, 4);

                item.Km = number_format(item.Km, 0, ',', '.');

                if (item.Price.Vat) {
                    item.Price.Net =  number_format(item.Price.Client / (1 + 19/100), 2, ',', '.');
                }

                item.Price.Client = number_format(item.Price.Client, 2, ',', '.');
            });

            return response;
        },

        getFilteredList: function() {

            return this.filter([
                'Id',
                'Pictures',
                // 'Category',
                'FullCategory',
                'Type',
                'Manufacturer.Name',
                'Model',
                'SModel',
                'RegDate',
                'Km',
                'Gearbox',
                'Power',
                'Fuel',
                'Price.Client',
                'Price.Vat',
                'FuelConsumption.Combined',
                'Co2',
                'Markets'
            ]);
        },

        addAccsItem: function(add_name) {

            if (false === _(this.get('BaseValue')).has('AccsList')) {
                this.get('BaseValue').AccsList = {Accs: []};
            }

            this.get('BaseValue').AccsList.Accs.push(add_name);

            return true;
        },

        removeAccsItem: function(remove_name) {

            if (!_(this.get('BaseValue').AccsList.Accs).isArray()) {
                return true;
            }

            this.get('BaseValue').AccsList.Accs = _(this.get('BaseValue').AccsList.Accs)
                .reject( function(item) {
                    return item == remove_name;
                });

            return true;
        }
    });

    var FilterView = Backbone.View.extend({

        className: 'filter_view',
        tagName: 'section',

        filter: new FilterModel(),

        initialize: function() {

            var that = this;

            _.bindAll(this, 'render', 'updateTotalCount');



            that.area           = new BaseFilterParam({name: 'Area',                label: 'Bereich', input:  'radio'});
            // that.category       = new BaseFilterParam({name: 'Category',            label: 'Kategorie'});
            that.manufacturer   = new BaseFilterParam({name: 'Manufacturer.Name',   label: 'Hersteller'});
            that.model          = new BaseFilterParam({name: 'Model',               label: 'Modell'});
            that.fuel           = new BaseFilterParam({name: 'Fuel',                label: 'Kraftstoff'});
            that.gearbox        = new BaseFilterParam({name: 'Gearbox',             label: 'Getriebe'});

            that.regdate        = new MinMaxFilterParam({name: 'RegDate',           label: 'EZ'});
            that.km             = new MinMaxFilterParam({name: 'Km',                label: 'Kilometerstand'});
            that.power          = new MinMaxFilterParam({name: 'Power',             label: 'Leistung'});
            that.price          = new MinMaxFilterParam({name: 'Price.Client',      label: 'Preis'});


            that.accs_list      = new AccsListView({items: ACCS_LIST});

            that.filter.on('grouped.Area', function(items) {
                that.area.refreshItems(items);
            });

            // that.filter.on('grouped.Category', function(items) {
            //     that.category.refreshItems(items);
                // that.trigger('loading.done');
            // });

            that.filter.on('grouped.Manufacturer', function(items) {
                that.manufacturer.refreshItems(items);
            });

            that.filter.on('reset.Manufacturer', function(items) {
                that.filter.reset('Model');
                that.model.refreshItems([]);
            });

            that.filter.on('grouped.Model', function(items) {
                that.model.refreshItems(items);
            });

            that.filter.on('grouped.Fuel', function(items) {
                that.fuel.refreshItems(items);
            });

            that.filter.on('grouped.Gearbox', function(items) {
                that.gearbox.refreshItems(items);
            });

            that.filter.on('grouped.RegDate', function(items) {
                that.regdate.refreshItems(items);
            });

            that.filter.on('grouped.Km', function(items) {
                that.km.refreshItems(items);
            });

            that.filter.on('grouped.Power', function(items) {
                that.power.refreshItems(items);
            });

            that.filter.on('grouped.Price.Client', function(items) {
                that.price.refreshItems(items);
            });

            that.filter.on('grouped.AccsList', function(available_accs_list) {

                // Get the list of values of all accessories.
                var max_accs_list = _(ACCS_LIST).map(function(item) {
                    return item.value;
                });

                // Get the list of values of accessories available by current filter.
                var intersected = _.intersection(available_accs_list, max_accs_list);

                that.accs_list.enableItems(intersected);
            });

            that.filter.on('filtered', function(result) {
                that.updateTotalCount(result.total);
                that.trigger('filtered', result);
            });

            that.area.on('change', function(value) {

                that.filter.reset('Manufacturer');

                that.filter.get('BaseValue').Area = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.manufacturer.on('change', function(value) {

                that.filter.get('BaseValue').Manufacturer = {
                    Id: value
                };


                that.filter.reset('Model');

                if (value == '0') {
                    that.model.refreshItems([]);
                }

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.model.on('change', function(value) {

                that.filter.get('BaseValue').Model = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.fuel.on('change', function(value) {

                that.filter.get('BaseValue').Fuel = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.gearbox.on('change', function(value) {

                that.filter.get('BaseValue').Gearbox = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.regdate.on('change', function(value, filter_type) {

                that.filter.get(filter_type).RegDate = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.km.on('change', function(value, filter_type) {

                that.filter.get(filter_type).Km = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.power.on('change', function(value, filter_type) {

                that.filter.get(filter_type).Power = value;

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.price.on('change', function(value, filter_type) {

                that.filter.get(filter_type).Price = {
                    Client: value
                };

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.accs_list.on('change.add', function(name) {
                that.filter.addAccsItem(name);
                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });

            that.accs_list.on('change.remove', function(name) {

                that.filter.removeAccsItem(name);

                that.filter.page_num = 1;
                that.filter.groupAll();
                that.filter.getFilteredList();
            });


            that.filter.group('Area');

            that.render();
        },

        render: function() {

            var that = this;

            var $base_params = $('<div class="base_params"></div>');
            var $minmax_params = $('<div class="minmax_params"></div>');

            var $total_count = $('<div id="total_count"><span class="total">0</span> Fahrzeug(e)</div>');

            $base_params.append(
                that.area.$el,
                // that.category.$el,
                that.manufacturer.$el,
                that.model.$el,
                that.fuel.$el,
                that.gearbox.$el
            );

            $minmax_params.append(
                that.regdate.$el,
                that.km.$el,
                that.power.$el,
                that.price.$el
            );

            this.$el.empty().append(
                $base_params,
                $minmax_params,
                that.accs_list.$el,
                $total_count
            );
        },

        updateTotalCount: function(total) {
            this.$el.find('#total_count .total').empty().append(total);
        }
    });

    /*
     * ------------------- V E H I C L E   L I S T
     */
    var VehicleListItemModel = Backbone.Model.extend({

        defaults: {
            Id: null,
            Pictures: null,
            Manufacturer: null,
            Model: null,
            SModel: null,
            RegDate: null,
            Km: null,
            Freetext: null
        }
    });

    var VehicleListItemView = Backbone.View.extend({

        tagName: 'li',
        className: 'vehicle',

        model: null,

        events: {
            'click': function() {
                this.trigger('click', this.$el.attr('vehicle-id'));
            }
        },

        initialize: function(options) {

            if (!_(options).has('model')) {
                throw new Error('No model given.');
            }

            this.model = options.model;

            _.bindAll(this, 'render');
            this.render();
        },

        render: function() {

            var that = this;

            $.get('assets/templates/vehicle_list_item.tpl.html', function(tpl) {
                that.$el
                    .empty()
                    .append(_.template(tpl, that.model.toJSON()))
                    .attr('vehicle-id', that.model.get('Id'));
            });

            return this;
        }
    });

    var VehicleListCollection = Backbone.Collection.extend({
        model: VehicleListItemModel
    });

    var VehicleListView = Backbone.View.extend({

        tagName: 'section',

        attributes: {
            id: 'vehicle_list'
        },

        items: new VehicleListCollection(),

        pagination: new PaginationView({limit: PAGIN_MAX_ITEMS_PER_PAGE}),

        initialize: function() {

            _.bindAll(this, 'render', 'update', 'refresh');

            var that = this;

            this.items.on('reset', function() {
                that.refresh();
            });

            this.pagination.on('select.page', function(pagination) {
                that.trigger('filter', pagination);
            });

            this.render();
        },

        render: function() {

            var that = this;

            this.$el
                .empty()
                .append(
                    this.pagination.$el,
                    $('<ul class="list"></ul>')
                );
        },

        refresh: function() {

            var that = this;
            var vehicle_list = [];

            _(this.items.models).each(function(model) {
                var vehicle = new VehicleListItemView({model: model});
                vehicle.on('click', function(id) {
                    that.trigger('click.vehicle', id);
                });

                vehicle_list.push(vehicle.$el);
            });

            this.$el.find('ul.list').empty().append(vehicle_list);

            return true;
        },

        update: function(update_data) {

            this.pagination.refresh({
                num: update_data.page_num,
                total: update_data.total
            });

            this.items.reset(update_data.items);

            return true;
        }
    });


    /*
     * ------------------- V E H I C L E   V I E W
     */
    var VehicleModel = Backbone.Model.extend({

        url: 'data/vehicle.php',

        initialize: function() {

        },

        parse: function(response) {

            response.PowerPs = Number(response.Power * 1.359621617).toFixed(0);
            response.RegDate = response.RegDate.substr(5, 2) + '/' + response.RegDate.substr(0, 4);
            response.GeneralInspection.Date = (response.GeneralInspection.Date == '0000-00-00')
                                                ? '-'
                                                : response.GeneralInspection.Date.substr(5, 2) + '/' + response.GeneralInspection.Date.substr(0, 4);

            response.Km = number_format(response.Km, 0, ',', '.');

            if (response.Price.Vat) {
                response.Price.Net =  number_format(response.Price.Client / (1 + 19/100), 2, ',', '.');
            }

            response.Price.Client = number_format(response.Price.Client, 2, ',', '.');

            response.Pictures = response.Pictures || null;

            this.trigger('parsed');
            return response;
        }
    });

    var VehicleView = Backbone.View.extend({

        vehicle: new VehicleModel(),

        attributes: {
            id: 'vehicle'
        },

        events: {

            /*
             * Pictures thumbnails
             */
            'click li': function(e) {
                var $target = $(e.target);
                this.$el.find('.pictures > img').attr('src', $target.attr('medium-pic'));
            }
        },

        initialize: function() {

            var that = this;
            _.bindAll(this, 'render');

            this.vehicle.on('parsed', function() {
                that.render();
            });
        },

        render: function() {

            var that = this;

            $.get('assets/templates/vehicle.tpl.html', function(tpl) {
                that.$el.empty().append(_.template(tpl, that.vehicle.toJSON()));
                that.trigger('rendered');
            });

            return this;
        }
    });

    var Button = Backbone.View.extend({

        name: null,
        className: 'button',

        events: {
            'click': function() {
                this.trigger('click', this.name);
            }
        },

        initialize: function(options) {
            _.bindAll(this, 'render');

            options = options || {};

            if (!_(options).has('name')) {
                throw new Error('No name given.');
            }

            this.name = options.name;
            this.text = _(options).has('text') ? options.text : this.name;

            this.render();
        },

        render: function() {

            this.$el
                .append(this.text)
                .attr('id', 'btn_' + this.name);

            return this;
        }
    });

    /*
     * ------------------- M A I N   V I E W
     */
     var MainView = Backbone.View.extend({

        filter_view:     new FilterView(),
        list_view:       new VehicleListView(),
        vehicle_view:    new VehicleView(),

        initialize: function() {

            var that = this;

            _.bindAll(this, 'render', 'showFilter', 'showList', 'showVehicle');

            this.btnBackToVehicleList = new Button({name: 'backto_vehicle_list', text: 'zur Liste'});
            this.btnBackToFilter = new Button({name: 'backto_filter', text: 'zur Suche'});
            this.btnShowVehicleList = new Button({name: 'show_vehicle_list', text: 'Trefferliste'});

            this.btnShowVehicleList.on('click', that.showList);
            this.btnBackToFilter.on('click', that.showFilter);
            this.btnBackToVehicleList.on('click', that.showList);

            this.filter_view.on('filtered', function(result) {
                that.list_view.update(result);
            });

            this.list_view.on('filter', function(pagination) {

                that.filter_view.filter.page_num = pagination.page_num;
                that.filter_view.filter.limit = pagination.limit;

                that.filter_view.filter.getFilteredList();
            });

            this.list_view.on('click.vehicle', function(vehicle_id) {

                _app.trigger('loading-started');
                that.vehicle_view.vehicle.fetch({
                    data: {
                        Id: vehicle_id
                    },
                    success: function(model) {
                        _app.trigger('loading-finished');
                    },
                    error: function() {
                        alert('Error fetching vehicle ' + vehicle_id);
                    }
                });
            });

            this.vehicle_view.on('rendered', function() {
                that.showVehicle();
            });

            this.render();
            this.showFilter();
        },

        render: function() {

            var that = this;

            var $header  = $('<section id="header"></section>');
            var $content = $('<section id="content"></section>');
            var $footer  = $('<section id="footer"></section>');

            var $heading = $('<h1>CarAPI Demo v2</h1><img id="isLoading" src="assets/pics/chase.gif" />');

            var $buttons = $('<div class="buttons"></div>');

            $buttons
                .append(
                    this.btnBackToFilter.$el,
                    this.btnBackToVehicleList.$el
                );

            $header
                .append(
                    $heading,
                    $buttons
                );

            $content.append(
                this.filter_view.$el,
                this.list_view.$el,
                this.vehicle_view.$el
            );

            $footer
                .append(
                    this.btnShowVehicleList.$el
                );

            this.$el
                .empty()
                .append($header, $content, $footer)
                .attr('id', 'app');

            return this;
        },

        showFilter: function() {

            this.filter_view.show();
            this.list_view.hide();
            this.vehicle_view.hide();

            this.btnBackToVehicleList.hide();
            this.btnBackToFilter.hide();
            this.btnShowVehicleList.show();
        },

        showList: function() {

            this.filter_view.hide();
            this.list_view.show();
            this.vehicle_view.hide();

            this.btnBackToVehicleList.hide();
            this.btnBackToFilter.show();
            this.btnShowVehicleList.hide();
        },

        showVehicle: function() {

            this.list_view.hide();
            this.filter_view.hide();
            this.vehicle_view.show();

            this.btnBackToVehicleList.show();
            this.btnBackToFilter.show();
            this.btnShowVehicleList.hide();
        }
    });

    /**
     * Show the app
     *
     * @return {boolean}
     */
    var _show = function() {

       var app_view = new MainView();

       $('body')
           .empty()
           .prepend(app_view.$el);

        return true;
    };

    _app.show = _show;

    return _app;

})();


/*
 * Run the app.
 */
$(function () {

    app.show();

});