PaginationModel = Backbone.Model.extend({

    defaults: {
        num:        1,
        limit:      5,
        pages:      0,
        total:      0,
        show_prev:  false,
        show_next:  true,
        first_item: 0,
        last_item:  0
    },

    initialize: function() {

        _.bindAll(this, 'reconfigure', 'get_next_page', 'get_prev_page');

        return true;
    },

    reconfigure: function() {

        this.set({
            pages: Math.ceil(this.get('total') / this.get('limit'))
        },
        {
            silent: true
        });

        /*
         * Calc number of first and last item for current page.
         */
        this.set({
            first_item: (this.get('num') - 1) * this.get('limit') + 1
        });

        this.set({
            last_item: (this.get('num') < this.get('pages'))
                        ? this.get('num') * this.get('limit')
                        : this.get('total')
        });

        /*
         * Enable/disable first, last, prev and next.
         */
        this.set({show_prev: (this.get('num') > 1)});
        this.set({show_next: (this.get('num') < this.get('pages'))});

        return this;
    },

    get_next_page: function() {

        var next_page = Number(this.get('num')) + 1;

        if (next_page > this.get('pages')) {
            next_page = this.get('pages');
        }

        return next_page;
    },

    get_prev_page: function() {

        var prev_page = Number(this.get('num')) - 1;
        if (prev_page < 1) {
            prev_page = 1;
        }

        return prev_page;
    }
});

PaginationView = Backbone.View.extend({

    className: 'pagination',

    model: new PaginationModel(),

    events: {
        'click a.prev_page':        'prev_page',
        'click a.next_page':        'next_page',
        'click a.page':             'select_page',
        'click option.page':        'select_page',
        'click option.page_limit':  'change_limit'
    },

    initialize: function(options) {

        _.bindAll(this, 'prev_page', 'next_page', 'select_page', 'refresh', 'render');

        options = options || {};

        if (_(options).has('limit')) {
            this.model.set({limit: options.limit});
        }

        return true;
    },

    render: function() {

        var that = this;

        $.get('assets/templates/pagination.tpl.html', function(template) {

            that.model.reconfigure();
            $(that.el).html(_.template(template)(that.model.toJSON()));

            return true;
        });

        return this;
    },

    refresh: function(pagin_data) {
        this.model.set(pagin_data);
        return this.render();
    },

    select_page: function(event) {

        $target = $(event.target);

        var page_num = 1;

        if ($target.is('a')) {
            page_num = Number($target.attr('page'));
        }
        else if ($target.is('option')) {
            page_num = Number($target.val());
        }

        this.model.set({
            num: page_num
        });

        this.trigger('select.page', {
            page_num: this.model.get('num'),
            limit: this.model.get('limit')
        });

        return true;
    },

    prev_page: function() {

        this.model.set({
            num: this.model.get_prev_page()
        });

        this.trigger('select.page', {
            page_num: this.model.get('num'),
            limit: this.model.get('limit')
        });

        return true;
    },

    next_page: function() {

        this.model.set({
            num: this.model.get_next_page()
        });

        this.trigger('select.page', {
            page_num: this.model.get('num'),
            limit: this.model.get('limit')
        });
        return true;
    },

    change_limit: function(event) {

        $target = $(event.target);

        this.model.set({
            num:   1,
            limit: Number($target.val())
        });

        this.trigger('select.page', {
            page_num: this.model.get('num'),
            limit: this.model.get('limit')
        });

        return true;
    }
});