<?php

ob_start();

try {

    require_once realpath(
        dirname(__FILE__) . DIRECTORY_SEPARATOR .
        '..' . DIRECTORY_SEPARATOR .
        '..' . DIRECTORY_SEPARATOR .
        'app' . DIRECTORY_SEPARATOR .
        'engine') . DIRECTORY_SEPARATOR . 'configure.inc.php';

    if (false === array_key_exists('filter', $_POST)) {
        throw new Exception('No filter in filter request.');
    }

    /*
     * Assoc array to stdClass object.
     */
    $filter = json_decode(json_encode($_POST['filter'], JSON_FORCE_OBJECT));

    if (false === array_key_exists('BaseValue', $filter)) {
        $filter->BaseValue = new stdClass();
    }

    /*
     * Bring AccsList to format.
     */
    if (array_key_exists('AccsList', $filter->BaseValue)) {

        $accs_list = [];

        foreach ($filter->BaseValue->AccsList->Accs as $accs_list_item) {
            array_push($accs_list, ['Key' => $accs_list_item, 'Value' => 1]);
        }

        $filter->BaseValue->AccsList->Accs = $accs_list;
    }


    /*
     * No sold or deleted vehicles.
     */
    $filter->BaseValue->State = new stdClass();
    $filter->BaseValue->State->Sold = 0;
    $filter->BaseValue->State->Deleted = 0;

    // error_log('['.__FILE__.'] Filter: ' . var_export($filter, true));

    /*
     * Pagination.
     */
    $page_num   = intval($_POST['pagination']['page_num'])  ? intval($_POST['pagination']['page_num'])  : 0;
    $limit      = intval($_POST['pagination']['limit'])     ? intval($_POST['pagination']['limit'])     : 30;

    $start = ($page_num - 1) * $limit;

    $order = 'CreateCreationDate';
    $direction = 'DESC';
    $filtered = CarApiClient :: filterVehicle($filter, array('Field' => $_POST['fields']), $start, $limit, $order, $direction);

    foreach($filtered['VehicleFilterResponse']->Vehicle as &$vehicle) {

        $vehicle->FullCategory  = CarApiClient :: translateValue($vehicle->FullCategory,    'FullCategory');
        $vehicle->Type          = CarApiClient :: translateValue($vehicle->Type,            'Type');
        $vehicle->Gearbox       = CarApiClient :: translateValue($vehicle->Gearbox,         'Gearbox');
        $vehicle->Fuel          = CarApiClient :: translateValue($vehicle->Fuel,            'Fuel');
    }

    echo json_encode(array(
        'items'     => $filtered['VehicleFilterResponse']->Vehicle,
        'total'     => $filtered['TotalCount'],
        'page_num'  => $page_num
    ));
}
catch (Exception $e) {

    error_log('[' . basename(__FILE__) .'] ' . $e->getMessage());

    header("HTTP/1.0 500 Internal Server Error");
    echo json_encode($e->getMessage());
}

ob_end_flush();
?>